#include <windows.h>
#include <stdio.h>
#include "myWindow.h"
#include "myResource.h"


extern HDC hMaskDC;
extern BITMAP bitmap;
extern HBITMAP hMaskBitmap;

PBITMAPINFO CreateBitmapInfoStruct(HBITMAP hBmp)
{ 
    BITMAP bmp; 
    PBITMAPINFO pbmi; 
    WORD    cClrBits; 

    // Retrieve the bitmap's color format, width, and height. 
    if (!GetObject(hBmp, sizeof(BITMAP), (LPSTR)&bmp)) 
	{
        MessageBox(NULL, "GetObject error", "error", MB_OK); 
	}

    // Convert the color format to a count of bits. 
    cClrBits = (WORD)(bmp.bmPlanes * bmp.bmBitsPixel); 
    if (cClrBits == 1) 
        cClrBits = 1; 
    else if (cClrBits <= 4) 
        cClrBits = 4; 
    else if (cClrBits <= 8) 
        cClrBits = 8; 
    else if (cClrBits <= 16) 
        cClrBits = 16; 
    else if (cClrBits <= 24) 
        cClrBits = 24; 
    else cClrBits = 32; 

    // Allocate memory for the BITMAPINFO structure. (This structure 
    // contains a BITMAPINFOHEADER structure and an array of RGBQUAD 
    // data structures.) 

     if (cClrBits != 24) 
         pbmi = (PBITMAPINFO) LocalAlloc(LPTR, 
                    sizeof(BITMAPINFOHEADER) + 
                    sizeof(RGBQUAD) * (1<< cClrBits)); 

     // There is no RGBQUAD array for the 24-bit-per-pixel format. 

     else 
         pbmi = (PBITMAPINFO) LocalAlloc(LPTR, 
                    sizeof(BITMAPINFOHEADER)); 

    // Initialize the fields in the BITMAPINFO structure. 

    pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
    pbmi->bmiHeader.biWidth = bmp.bmWidth; 
    pbmi->bmiHeader.biHeight = bmp.bmHeight; 
    pbmi->bmiHeader.biPlanes = bmp.bmPlanes; 
    pbmi->bmiHeader.biBitCount = bmp.bmBitsPixel; 
    if (cClrBits < 24) 
        pbmi->bmiHeader.biClrUsed = (1<<cClrBits); 

    // If the bitmap is not compressed, set the BI_RGB flag. 
    pbmi->bmiHeader.biCompression = BI_RGB; 

    // Compute the number of bytes in the array of color 
    // indices and store the result in biSizeImage. 
    // For Windows NT/2000, the width must be DWORD aligned unless 
    // the bitmap is RLE compressed. This example shows this. 
    // For Windows 95/98, the width must be WORD aligned unless the 
    // bitmap is RLE compressed.
    pbmi->bmiHeader.biSizeImage = ((pbmi->bmiHeader.biWidth * cClrBits +31) & ~31) /8
                                  * pbmi->bmiHeader.biHeight; 
    // Set biClrImportant to 0, indicating that all of the 
    // device colors are important. 
     pbmi->bmiHeader.biClrImportant = 0; 
     return pbmi; 
} 


void readFile(char* fileName)
{
	FILE* file;
	BITMAPFILEHEADER fileHeader;
	BITMAPINFOHEADER fileInfo;
	unsigned char byte;
	file=fopen(fileName, "r");
	fread(&fileHeader, sizeof(BITMAPFILEHEADER), 1, file);
	fread(&fileInfo, sizeof(BITMAPINFOHEADER), 1, file);
	fseek(file, fileHeader.bfOffBits, SEEK_SET);
	for (unsigned int i=0; i<fileInfo.biSizeImage; i++)
	{
		fread(&byte, 1, 1, file);
		if (byte!=255)
		{
			break;
		}
	}
	fclose(file);
}



void CreateBMPFile(LPTSTR pszFile, PBITMAPINFO pbi, 
                  HBITMAP hBMP, HDC hDC) 
 { 
    HANDLE hf;                 // file handle 
    BITMAPFILEHEADER hdr;       // bitmap file-header 
    PBITMAPINFOHEADER pbih;     // bitmap info-header 
    LPBYTE lpBits;              // memory pointer 
    DWORD dwTotal;              // total count of bytes 
    DWORD cb;                   // incremental count of bytes 
    BYTE *hp;                   // byte pointer 
    DWORD dwTmp; 
	int result;
	RGBQUAD *colorTable;

    pbih = (PBITMAPINFOHEADER) pbi; 
    lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);

    if (!lpBits) 
	{
		MessageBox(NULL, "GlobalAlloc error", "error", MB_OK); 
	}

    // Retrieve the color table (RGBQUAD array) and the bits 
    // (array of palette indices) from the DIB. 
	
    if (!(result=GetDIBits(hDC, hBMP, 0, (WORD) pbih->biHeight, lpBits, pbi, 
    //    DIB_RGB_COLORS))) 
			DIB_PAL_COLORS)))
    {
        MessageBox(NULL, "GetDIBits", "error", MB_OK); 
    }
	if (pbi->bmiHeader.biBitCount==1)
	{
		pbi->bmiHeader.biClrUsed=2;	
		colorTable=new RGBQUAD[2];
		colorTable[0].rgbRed=255;
		colorTable[0].rgbGreen=0;
		colorTable[0].rgbBlue=0;
		colorTable[0].rgbReserved=0;
		colorTable[1].rgbRed=0;
		colorTable[1].rgbGreen=255;
		colorTable[1].rgbBlue=0;
		colorTable[1].rgbReserved=0;	
	}
	


    // Create the .BMP file. 
    hf = CreateFile(pszFile, 
                   GENERIC_READ | GENERIC_WRITE, 
                   (DWORD) 0, 
                    NULL, 
                   CREATE_ALWAYS, 
                   FILE_ATTRIBUTE_NORMAL, 
                   (HANDLE) NULL); 
    if (hf == INVALID_HANDLE_VALUE) 
	{
		MessageBox(NULL, "CreateFile", "error", MB_OK); 
	}
    hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
    // Compute the size of the entire file. 
    hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
                 pbih->biSize + pbih->biClrUsed 
                 * sizeof(RGBQUAD) + pbih->biSizeImage); 
    hdr.bfReserved1 = 0; 
    hdr.bfReserved2 = 0; 

    // Compute the offset to the array of color indices. 
    hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
                    pbih->biSize + pbih->biClrUsed 
                    * sizeof (RGBQUAD); 

    // Copy the BITMAPFILEHEADER into the .BMP file. 
    if (!WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER), 
        (LPDWORD) &dwTmp,  NULL)) 
    {
        MessageBox(NULL, "WriteFile", "error", MB_OK); 
    }

    // Copy the BITMAPINFOHEADER and RGBQUAD array into the file. 
    if (!WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER) , 
                  (LPDWORD) &dwTmp, ( NULL))) 
	{
         MessageBox(NULL, "WriteFile error", "Error", MB_OK); 
	}

	if (!WriteFile(hf, (LPVOID) colorTable, pbih->biClrUsed * sizeof (RGBQUAD), 
                  (LPDWORD) &dwTmp, ( NULL))) 
	{
         MessageBox(NULL, "WriteFile error", "Error", MB_OK); 
	}
	

    // Copy the array of color indices into the .BMP file. 
    dwTotal = cb = pbih->biSizeImage; 
    hp = lpBits; 
    if (!WriteFile(hf, (LPSTR) lpBits, (int) cb, (LPDWORD) &dwTmp,NULL)) 
	{
           MessageBox(NULL, "WriteFile", "error", MB_OK);
	}

    // Close the .BMP file. 
     if (!CloseHandle(hf)) 
	 {
           MessageBox(NULL, "CloseHandle", "error", MB_OK); 
	 }

    // Free memory. 
    GlobalFree((HGLOBAL)lpBits);
	readFile("mask.bmp");
}



void doSaveScreen(HDC hdc, RECT& rect, char* fileName)
{
	HDC hdcMem;
	HBITMAP hMyBitmap;
	PBITMAPINFO pBitmapInfo;
	POINT point;

	hdcMem=CreateCompatibleDC(hdc);
	hMyBitmap=CreateCompatibleBitmap(hdc, rect.right-rect.left, rect.bottom-rect.top);
	

	SelectObject(hdcMem, hMyBitmap);
	
	
	BitBlt(hdcMem, 0, 0, rect.right-rect.left, rect.bottom-rect.top, hdc, 
		rect.left, rect.top, SRCCOPY);

	pBitmapInfo=CreateBitmapInfoStruct(hMyBitmap);
	CreateBMPFile(fileName, pBitmapInfo, hMyBitmap, hdcMem);

	DeleteDC(hdcMem);
	DeleteObject(hMyBitmap);
	LocalFree(pBitmapInfo);

}

void saveScreen(HWND hwnd)
{
	int xScreen, yScreen;
	RECT rect;
	HDC hdc;
	
	hdc=CreateDC("DISPLAY", NULL, NULL, NULL);
	rect.left=rect.top=0;
	rect.right=GetDeviceCaps(hdc, HORZRES);
	rect.bottom=GetDeviceCaps(hdc, VERTRES);
	doSaveScreen(hdc, rect, "fullScreen.bmp");
	DeleteDC(hdc);
	
}

void saveClientArea(HWND hwnd)
{
	HDC hdc;
	POINT origin;
	RECT rect;
	hdc=GetWindowDC(hwnd);
	clientAreaOrg(origin, hwnd);
	GetWindowRect(hwnd, &rect);
	rect.left+=origin.x;
	rect.top+=origin.y;
	doSaveScreen(hdc, rect, "clientArea.bmp");
	ReleaseDC(hwnd, hdc);
}

void saveMask(HWND hwnd)
{
	PBITMAPINFO pBitmapInfo;
	pBitmapInfo=CreateBitmapInfoStruct(hMaskBitmap);
	CreateBMPFile("mask.bmp", pBitmapInfo, hMaskBitmap, hMaskDC);
	LocalFree(pBitmapInfo);
}


long CALLBACK getScreenShot(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_COMMAND:
		switch (wParam)
		{
		case IDM_FULLSCREENSHOT:
			saveScreen(hwnd);
			break;
		case IDM_CLIENTAREASHOT:
			saveClientArea(hwnd);
			break;
		case  IDM_MASKSHOT:
			saveMask(hwnd);
			break;
		}
		break;
	}
	return 0L;
}
		

