#include <windows.h>
#include "./include/ijl.h"
#include "jpegLoader.h"
#include "errHandle.h"
#pragma comment(lib, "./lib/ijl15.lib")


bool loadJPEG(char* fileName, BITMAP& bitmap)
{
	 //Error handle
	IJLERR jerr;
	//this maybe good for multiple threads
	//Create the JPEG object
	JPEG_CORE_PROPERTIES jcprops;

	//Initialize the JPEG object
	jerr = ijlInit(&jcprops);
	if (jerr != IJL_OK)
	{
		MessageBox(NULL, "IJLInit problem", "init failed", MB_OK);
		return false;
	}
	
		//Set the IJL data source as a filename
	jcprops.JPGFile = fileName;

	//Read JPEG parameters from the file
	jerr = ijlRead(&jcprops, IJL_JFILE_READPARAMS);

	//Make sure Parameter read was successful
	if (jerr != IJL_OK)
	{
		MessageBox(NULL, "Error reading JPEG parameters", "my box", MB_OK);
		return false;
	}
	bitmap.bmWidth=jcprops.JPGWidth;
	bitmap.bmHeight=jcprops.JPGHeight;
	bitmap.bmBitsPixel=24;
	bitmap.bmType=0;
	bitmap.bmPlanes=1;
	bitmap.bmWidthBytes=(jcprops.JPGWidth * 3+sizeof(int)-1)/sizeof(int)*sizeof(int);


	//Determine the required size
	long szbuff24 = bitmap.bmWidthBytes * jcprops.JPGHeight;

	//Resize the buffer

	bitmap.bmBits = new BYTE [szbuff24];
	if (bitmap.bmBits == NULL)
	{
		MessageBox(NULL, "Memory Allocation Error", "my box", MB_OK);
		return false;
	}

	//Set up the DIB specification for the JPEG decoder
	jcprops.DIBWidth    = jcprops.JPGWidth;
	jcprops.DIBHeight   = -jcprops.JPGHeight; //Implies a bottom-up DIB.
	jcprops.DIBChannels = 3;
	jcprops.DIBColor    = IJL_BGR;
	jcprops.DIBPadBytes = IJL_DIB_PAD_BYTES(jcprops.JPGWidth,3);
	jcprops.DIBBytes    = reinterpret_cast<BYTE*>(bitmap.bmBits);

	//Set the JPG color space ... this will always be somewhat of an
	//educated guess at best because JPEG is "color blind" (i.e.,
	//nothing in the bit stream tells you what color space the data was
	//encoded from. However, in this example we assume that we are
	//reading JFIF files which means that 3 channel images are in the
	//YCbCr color space and 1 channel images are in the Y color space.
	switch(jcprops.JPGChannels)
	{
	case 1: jcprops.JPGColor = IJL_G;
		break;

	case 3: jcprops.JPGColor = IJL_YCBCR;
		break;

	default:
	//This catches everything else, but no color twist will be
	//performed by the IJL.
		jcprops.DIBColor = (IJL_COLOR)IJL_OTHER;
		jcprops.JPGColor = (IJL_COLOR)IJL_OTHER;
		break;
	}
    
	jerr = ijlRead(&jcprops, IJL_JFILE_READWHOLEIMAGE);

  
	//Make sure the read was successful
	if (jerr != IJL_OK)
	{
		MessageBox(NULL, "Error reading JPEG image", "my box", MB_OK);
		return false;
	}

	jerr=ijlFree(&jcprops);

	if (jerr != IJL_OK)
	{
		MessageBox(NULL, "Error reading JPEG image", "my box", MB_OK);
		return false;
	}
	return true;
}


HBITMAP loadJPEGImage(HDC hdc, char*fileName, BITMAP& bitmap)
{
	HBITMAP hBitmap;
	BITMAPINFO bitmapInfo;
	
	if (loadJPEG(fileName, bitmap))
	{
		SAFE_CALL((hBitmap=CreateCompatibleBitmap(hdc, bitmap.bmWidth, bitmap.bmHeight)), 
			"CRATE BITMAP");
		
		bitmapInfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		bitmapInfo.bmiHeader.biWidth=bitmap.bmWidth;
		bitmapInfo.bmiHeader.biHeight=bitmap.bmHeight;

		bitmapInfo.bmiHeader.biPlanes=1;
		bitmapInfo.bmiHeader.biBitCount=24;

		bitmapInfo.bmiHeader.biCompression=BI_RGB;
		bitmapInfo.bmiHeader.biSizeImage=bitmap.bmWidthBytes*bitmap.bmHeight;
		//bitmapInfo.bmiHeader.biSizeImage=0;
		bitmapInfo.bmiHeader.biXPelsPerMeter=bitmapInfo.bmiHeader.biYPelsPerMeter=0;
		bitmapInfo.bmiHeader.biClrUsed=0;
		bitmapInfo.bmiHeader.biClrImportant=0;
		SAFE_CALL(SetDIBits(hdc, hBitmap, 0, bitmap.bmHeight, bitmap.bmBits, &bitmapInfo, 
			DIB_RGB_COLORS), "SET BITS FOR BITMAP");
		delete[]bitmap.bmBits;
		
	}
	else
	{
		hBitmap=NULL;
	}
	return hBitmap;
}